{
  This file was automatically generated on 2012-10-12.

  If you have a bugfix for this file and want to commit it,
  please fix the bug in the generator. You can find a link
  to the generator git on tinkerforge.com
}

unit BrickletPiezoBuzzer;

{$ifdef FPC}{$mode OBJFPC}{$H+}{$endif}

interface

uses
  Device, IPConnection, LEConverter;

const
  BRICKLET_PIEZO_BUZZER_FUNCTION_BEEP = 1;
  BRICKLET_PIEZO_BUZZER_FUNCTION_MORSE_CODE = 2;
  BRICKLET_PIEZO_BUZZER_CALLBACK_BEEP_FINISHED = 3;
  BRICKLET_PIEZO_BUZZER_CALLBACK_MORSE_CODE_FINISHED = 4;

type
  TBrickletPiezoBuzzerNotifyBeepFinished = procedure of object;
  TBrickletPiezoBuzzerNotifyMorseCodeFinished = procedure of object;

  /// <summary>
  ///  Device for controlling a piezo buzzer
  /// </summary>
  TBrickletPiezoBuzzer = class(TDevice)
  private
    beepFinishedCallback: TBrickletPiezoBuzzerNotifyBeepFinished;
    morseCodeFinishedCallback: TBrickletPiezoBuzzerNotifyMorseCodeFinished;
  protected
    procedure CallbackWrapperBeepFinished(const packet: TByteArray); virtual;
    procedure CallbackWrapperMorseCodeFinished(const packet: TByteArray); virtual;
  public
    /// <summary>
    ///  Creates an object with the unique device ID <c>uid</c>. This object can
    ///  then be added to the IP connection.
    /// </summary>
    constructor Create(const uid_: string);

    /// <summary>
    ///  Beeps with the duration in ms. For example: If you set a value of 1000,
    ///  the piezo buzzer will beep for one second.
    /// </summary>
    procedure Beep(const duration: longword); virtual;

    /// <summary>
    ///  Sets morse code that will be played by the piezo buzzer. The morse code
    ///  is given as a string consisting of "." (dot), "-" (minus) and " " (space)
    ///  for *dits*, *dahs* and *pauses*. Every other character is ignored.
    ///  
    ///  For example: If you set the string "...---...", the piezo buzzer will beep
    ///  nine times with the durations "short short short long long long short 
    ///  short short".
    ///  
    ///  The maximum string size is 60.
    /// </summary>
    procedure MorseCode(const morse: string); virtual;

    /// <summary>
    ///  This callback is triggered if a beep set by <see cref="BrickletPiezoBuzzer.TBrickletPiezoBuzzer.Beep"/> is finished
    /// </summary>
    property OnBeepFinished: TBrickletPiezoBuzzerNotifyBeepFinished read beepFinishedCallback write beepFinishedCallback;

    /// <summary>
    ///  This callback is triggered if the playback of the morse code set by
    ///  <see cref="BrickletPiezoBuzzer.TBrickletPiezoBuzzer.MorseCode"/> is finished.
    /// </summary>
    property OnMorseCodeFinished: TBrickletPiezoBuzzerNotifyMorseCodeFinished read morseCodeFinishedCallback write morseCodeFinishedCallback;
  end;

implementation

constructor TBrickletPiezoBuzzer.Create(const uid_: string);
begin
  inherited Create(uid_);
  expectedName := 'Piezo Buzzer Bricklet';
  bindingVersion[0] := 1;
  bindingVersion[1] := 0;
  bindingVersion[2] := 0;
  callbackWrappers[BRICKLET_PIEZO_BUZZER_CALLBACK_BEEP_FINISHED] := {$ifdef FPC}@{$endif}CallbackWrapperBeepFinished;
  callbackWrappers[BRICKLET_PIEZO_BUZZER_CALLBACK_MORSE_CODE_FINISHED] := {$ifdef FPC}@{$endif}CallbackWrapperMorseCodeFinished;
end;

procedure TBrickletPiezoBuzzer.Beep(const duration: longword);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_PIEZO_BUZZER_FUNCTION_BEEP, 8);
  LEConvertUInt32To(duration, 4, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletPiezoBuzzer.MorseCode(const morse: string);
var request: TByteArray;
begin
  request := CreateRequestPacket(stackID, BRICKLET_PIEZO_BUZZER_FUNCTION_MORSE_CODE, 64);
  LEConvertStringTo(morse, 4, 60, request);
  SendRequestNoResponse(request);
end;

procedure TBrickletPiezoBuzzer.CallbackWrapperBeepFinished(const packet: TByteArray);
begin
  Assert(packet <> nil); { Avoid 'Parameter not used' warning }
  if (Assigned(beepFinishedCallback)) then begin
    beepFinishedCallback();
  end;
end;

procedure TBrickletPiezoBuzzer.CallbackWrapperMorseCodeFinished(const packet: TByteArray);
begin
  Assert(packet <> nil); { Avoid 'Parameter not used' warning }
  if (Assigned(morseCodeFinishedCallback)) then begin
    morseCodeFinishedCallback();
  end;
end;

end.
